@@ -116,6 +116,14 @@ class Exchange:
         """Returns True if this exchange represents a chain."""
         return self._vertices[0].isNdd()
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Exchange):
+            return NotImplemented
+        return self.id == other.id
+
+    def __neq__(self, other: object) -> bool:
+        return not (self == other)
+
     def __len__(self):
         """Returns the number of vertices (donors) in the exchange. Note that
         in the case of a non-directed altruistic donor, this will be greater
@@ -169,6 +177,19 @@ class Exchange:
             prev_v = self[(ind - 1) % len(self)]
             prev_donor = prev_v.donor
             donor = v.donor
+            # Backarcs are tricky to deal with as if there is an NDD, we cannot
+            # access its recipient.
+            if prev_donor.NDD:
+                # Need to find exchange containing backarc back to NDD
+                recipients = set([donor.recipient])
+            else:
+                if donor.NDD:
+                    # Need to find backarc from NDD to previous recipient
+                    recipients = set([prev_donor.recipient])
+                else:
+                    # No NDDs involved, just look for exchange with same two
+                    # recipents
+                    recipients = set([donor.recipient, prev_donor.recipient])
             # Note that a backarc may exist in an alternate (rather than
             # embedded) exchange if this exchange is a chain, as an exchange is
             # an alternate if it has the same set of recipients (unlike
@@ -177,15 +198,10 @@ class Exchange:
             # the two vertices representing directed (paired) donors.
             if self.chain:
                 for exchange in self.alternates:
-                    if len(exchange) != 2:
-                        continue
-                    donors = [v.donor for v in exchange]
-                    if donor in donors and prev_donor in donors:
+                    if set(exchange.all_recipients()) == recipients:
                         backarc_exchanges.append(exchange)
-            # By definition, an embedded exchange has length two here.
             for exchange in self.embedded:
-                donors = [v.donor for v in exchange]
-                if donor in donors and prev_donor in donors:
+                if set(exchange.all_recipients()) == recipients:
                     backarc_exchanges.append(exchange)
         return backarc_exchanges
 
